# ======================================================================================
#  bE_sistematicNodeVisitors: specialized iterators for bE nodes...
# ======================================================================================
import hou
import bE_general
import nodeGetters
import houIterators

# ======================================================================================
#  ExternalSistematicNodeVisitor:
#    Visits every node in the current network and performs some operation on the nodes.
#    The operation should be defined through the subclasses.
# ======================================================================================
class ExternalSistematicNodeVisitor:
	def __init__(self, network=None, startingNodes = None):
		if not network:
			self.parent = hou.selectedNodes()[0].parent()
		else:
			self.parent = network
		self.startingNodes = startingNodes
	def visitAllNodes(self):
		if not self.startingNodes:
			selectedNodes = nodeGetters.getLeafNodes(self.parent).get()
		else:
			selectedNodes = self.startingNodes
		it = houIterators.depthFirstSearchIterator(	houIterators.ascendingConnexion(),
													self.parent,
													selectedNodes)
		for depth, node in it.navigate():
			self.dispatch(depth, node)
		self.close()
	def dispatch(self, depth, node):
	    className = node.type().name()
	    visitingMethod = getattr(self, 'visit' + className, self.visit)
	    return visitingMethod(depth, node)
	def visit(self, depth, node):
		print "Visiting:", node.name(), "of type:",node.type().name(), "at depth:", depth

# ======================================================================================
#  Closing visitor:
#    Useful for generating the final Object Merge node from all products not 
#    linked to any rule...
# ======================================================================================
class ClosingVisitor(ExternalSistematicNodeVisitor):
	def __init__(self, network=None):
		ExternalSistematicNodeVisitor.__init__(self, network)
		self.finalBuilding = hou.node(self.parent.path()+'/finalBuilding')
		if not self.finalBuilding:
			self.finalBuilding = self.parent.createNode('object_merge')
			self.finalBuilding.setName('finalBuilding')
		self.finalBuilding.parm('numobj').set(0)
		self.finalBuilding.moveToGoodPosition()
		# Creates a null node used only to avoid unnceessary cooking calculations...
		self.sillyNullNode = self.parent.createNode('null')
		self.sillyNullNode.setDisplayFlag(True)
		print
		print "ClosingVisitor -> generating final Object Merge content"
	def addNode(self, aNode, product):
		numobj = self.finalBuilding.parm('numobj').eval() + 1
		self.finalBuilding.parm('numobj').set(numobj)
		print 'Adding to closure: ' + aNode.path() + " => " + product
		self.finalBuilding.parm('objpath'+str(numobj)).set(aNode.path())
		self.finalBuilding.parm('group'+str(numobj)).set(product)	
	def visit(self, depth, aNode):
		print "VISITING: " + aNode.name() + ": " + aNode.type().name()
		if not bE_general.is_bE_node(aNode):
			print depth * "-" + aNode.name() + " is not buildingEngine (" + aNode.type().name() + ")"
			return True
		allProds = bE_general.products(aNode)
		effectiveProds = bE_general.effectivelyUsedProducts(self.parent, aNode)
		print "Visiting: ", aNode.name(), " of type ", aNode.type().name(), " => prods:", allProds, "(effective:", effectiveProds, ")"
		if effectiveProds != []:
			for prod in allProds:
				if prod not in effectiveProds and prod != '':
					self.addNode(aNode, prod)
		else:
			if len(aNode.outputs()) == 0:
				self.addNode(aNode, '')
		"""
		counter = bE_general.numProds(aNode)
		print "  evaluating", counter, "elements"
		for s in range(counter):
			product = aNode.parm('product'+str(s)).evalAsString()
			found = False
			for n in aNode.outputs():
				if product in n.parm('filter').evalAsString().split(' '):
					print '  ', s, ":", product, "used in", n.path()
					found = True
			if not found:
				numobj = self.finalBuilding.parm('numobj').eval() + 1
				self.finalBuilding.parm('numobj').set(numobj)
				print '  ', s, 'Adding to closure: ' + aNode.path() + " => " + product
				self.finalBuilding.parm('objpath'+str(numobj)).set(aNode.path())
				self.finalBuilding.parm('group'+str(numobj)).set(product)
		"""
		return True
		
	def close(self):
		self.finalBuilding.setDisplayFlag(True)
		self.sillyNullNode.destroy() #ok, we don't need it anymore...

"""
# ======================================================================================
#  pressAllButtonsVisitor:
#    Useful for... well... for calling the press button of all Subdiv nodes in the scene
# ======================================================================================
class pressAllButtonsVisitor(ExternalSistematicNodeVisitor):
	def __init__(self, network=None):
		ExternalSistematicNodeVisitor.__init__(self, network)
	def visitSubdiv(self, node):
		print "Pressing Button at:", node.name(), "of type:", node.type().name()
		node.parm('rel0').pressButton()
	def close(self):
		pass
"""
