"""
################################################################
#                                                              #
#              Topological Sort for Houdini                    #
#                                                              #
################################################################
"""

"""
From http://en.wikipedia.org/wiki/Topological_sorting:
------------------------------------------------------
In graph theory, a topological sort or topological ordering of a 
directed acyclic graph (DAG) is a linear ordering of its nodes in which each 
node comes before all nodes to which it has outbound edges. Every DAG has 
one or more topological sorts.

More formally, define the reachability relation R over the nodes of the 
DAG such that xRy if and only if there is a directed path from x to y. 
Then, R is a partial order, and a topological sort is a linear extension of 
this partial order, that is, a total order compatible with the partial order.

...

An alternative algorithm for topological sorting is based on depth-first search. 
For this algorithm, edges point in the opposite direction as the previous algorithm 
(and the opposite direction to that shown in the diagram in the Examples section 
above). There is an edge from x to y if job x depends on job y (in other words, 
if job y must be completed before job x can be started). The algorithm loops 
through each node of the graph, in an arbitrary order, initiating a depth-first 
search that terminates when it hits any node that has already been visited since 
the beginning of the topological sort:

L <- Empty list that will contain the sorted nodes
S <- Set of all nodes
function visit(node n)
    if n has not been visited yet then
        mark n as visited
        for each node m with an edge from n to m do
            visit(m)
        add n to L
for each node n in S do
    visit(n)

Note that each node n gets added to the output list L only after considering all 
other nodes on which n depends (all descendant nodes of n in the graph). 
Specifically, when the algorithm adds node n, we are guaranteed that all nodes 
on which n depends are already in the output list L: they were added to L either 
by the preceding recursive call to visit(), or by an earlier call to visit(). 
Since each edge and node is visited once, the algorithm runs in linear time. 
Note that the simple pseudocode above cannot detect the error case where the 
input graph contains cycles. The algorithm can be refined to detect cycles by 
watching for nodes which are visited more than once during any nested sequence
of recursive calls to visit() (e.g., by passing a list down as an extra argument 
to visit(), indicating which nodes have already been visited in the current call 
stack). This depth-first-search-based algorithm is the one described by Cormen, 
Leiserson & Rivest (1990); it seems to have been first described in print by 
Tarjan (1976).
"""
import hou

# This code only follows the links between nodes, does not take into account 
# expressions or stuff like that...
def visit(n, visitedList, L):
	if n == None: return
	if n.path() not in visitedList:
		visitedList.append(n.path())
		reversedEdges = n.inputs()
		for m in reversedEdges:
			visit(m, visitedList, L)
		L.append(n)

def topologicalConnectionSort(parent):
	visitedList = []
	L = []
	S = parent.children()
	for n in S:
		visit(n,visitedList,L)
	return L
